/*
    See the JupyterLab Developer Guide for useful CSS Patterns:

    https://jupyterlab.readthedocs.io/en/stable/developer/css.html
*/

.cell-tag-celltagsclasses-test1 {
  padding: 10px;
  border: 2.5px black solid;
  border-radius: 10px;

  /* could be overridden if cell is active */
  background-color: yellow !important;
}

.cell-tag-celltagsclasses-test2 {
  /* see how it goes if cell is active */
  background-color: red;
}

/* hide the 6-buttons toolbar */
.cell-tag-celltagsclasses-test2 .jp-cell-toolbar {
  display: none;
}
